; MidiPWMLatch
; July 2005
; by Johannes Taelman
;
; Specs:
; - controlling 8 latches with 8 outputs with note-on / note-off
; - 100% duty cycle pulse of fixed duration at note-on,
;	25% duty cycle afterwards until note-off
; - FSR2 is reserved for high-priority interrupt!
; - rough timer per note
; 
; DOING:
; - aivoid ISR reentrance

		list p=18f252		;list directive to define processor
		#include <p18f252.inc>	;processor specific definitions

; #define simulate 1 ; enable simulation
; #define heartbeat 1 ; enable heatbeats
;******************************************************************************

#define	ErrorLed	LATC,	0;
#define	ClearErrorLed	BCF	ErrorLed
#define	SetErrorLed		BSF	ErrorLed

		#include "..\midi.asm"

;Configuration bits

	CONFIG	OSC = HSPLL
	CONFIG	OSCS = OFF
	CONFIG	PWRT = ON
	CONFIG	BOR = ON	
	CONFIG	BORV = 27	
	CONFIG	WDT = OFF
	CONFIG	WDTPS = 1
	CONFIG	CCP2MUX = ON
	CONFIG	STVR = ON
	CONFIG	LVP = OFF

;----------------------------------------------------------------------------
; Variables
; memory map 
;----------------------------------------------------------------------------

		CBLOCK	0x000
		WREG_TEMP		;to save WREG during interrupt
		STATUS_TEMP		;to save STATUS during interrupt
		BSR_TEMP		;to save BSR during interrupt
		FSR0H_TEMP		;to save FSR0H during interrupt
		FSR0L_TEMP		;to save FSR0L during interrupt
		TBLPTRH_TEMP
		TBLPTRL_TEMP
		TABLAT_TEMP
		FSR0H_SHADOW		;to save FSR0H during high interrupt
		FSR0L_SHADOW		;to save FSR0L during high interrupt
		TmpM			; non-ISR scratch
		TmpT			; non-ISR scratch		
		LatWriteState	;	0=do nothing
						;	1=set data
						;	2=set addr
						;	2=release addr
		NoteCounter		; cycles through notes		
		FifoRdIndx;
		FifoWrIndx;
		TISRData;
		TISRAddrA;
		TISRAddrC;
		ENDC

		CBLOCK 0x200	; This datastructure is in the order of execution 
						; and thus implies the duty-cycle...	

		ExtLatAData1	; hold bits (25% duty cycle)
		ExtLatAStrobeA1
		ExtLatAStrobeC1
		dummy1

		ExtLatHData2	; attack bits (75% duty cycle)
		ExtLatHStrobeA2
		ExtLatHStrobeC2
		dummy2

		ExtLatBData1	; hold bits (25% duty cycle)
		ExtLatBStrobeA1
		ExtLatBStrobeC1
		dummy3
		
		ExtLatIData2	; attack bits (75% duty cycle)
		ExtLatIStrobeA2
		ExtLatIStrobeC2
		dummy4

		ExtLatCData1	; hold bits (25% duty cycle)
		ExtLatCStrobeA1
		ExtLatCStrobeC1
		dummy5

		ExtLatAData2	; attack bits (75% duty cycle)
		ExtLatAStrobeA2
		ExtLatAStrobeC2
		dummy6

		ExtLatDData1	; hold bits (25% duty cycle)
		ExtLatDStrobeA1
		ExtLatDStrobeC1
		dummy7

		ExtLatBData2	; attack bits (75% duty cycle)
		ExtLatBStrobeA2
		ExtLatBStrobeC2
		dummy8

		ExtLatEData1	; hold bits (25% duty cycle)
		ExtLatEStrobeA1
		ExtLatEStrobeC1
		dummy9

		ExtLatCData2	; attack bits (75% duty cycle)
		ExtLatCStrobeA2
		ExtLatCStrobeC2
		dummy10
		
		ExtLatFData1	; hold bits (25% duty cycle)
		ExtLatFStrobeA1
		ExtLatFStrobeC1
		dummy11

		ExtLatDData2	; attack bits (75% duty cycle)
		ExtLatDStrobeA2
		ExtLatDStrobeC2
		dummy12
		
		ExtLatGData1	; hold bits (25% duty cycle)
		ExtLatGStrobeA1
		ExtLatGStrobeC1
		dummy13

		ExtLatEData2	; attack bits (75% duty cycle)
		ExtLatEStrobeA2
		ExtLatEStrobeC2
		dummy14

		ExtLatHData1	; hold bits (25% duty cycle)
		ExtLatHStrobeA1
		ExtLatHStrobeC1
		dummy15

		ExtLatFData2	; attack bits (75% duty cycle)
		ExtLatFStrobeA2
		ExtLatFStrobeC2
		dummy16
		
		ExtLatIData1	; hold bits (25% duty cycle)
		ExtLatIStrobeA1
		ExtLatIStrobeC1
		dummy17

		ExtLatGData2	; attack bits (75% duty cycle)
		ExtLatGStrobeA2
		ExtLatGStrobeC2
		dummy18

		ExtLatDummyData1	
		ExtLatDummyStrobeA1
		ExtLatDummyGStrobeC1
		dummy19
		
		ExtLatDummyData2	
		ExtLatDummyStrobeA2
		ExtLatDummyGStrobeC2
		dummy20
		ENDC

#define Wrap1 ExtLatDummyData1


		CBLOCK	0x100	; fifo from main to timer ISR
		NoteTimer:80	; timer counter
		ENDC

;----------------------------------------------------------------------------
; end of memory map
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;This code executes when a reset occurs.

		ORG     0x0000		;place code at reset vector

ResetVector:			bra	Main		;go to beginning of program

;----------------------------------------------------------------------------
;This code executes when a high priority interrupt occurs.

		ORG	0x0008

HighInt:
		btfsc	PIE2,TMR3IE	;else test if Timer3 interrupt enabled
		bra		Timer3ISR		;if so, go get received data
		reset			;error if no valid interrupt so reset

;----------------------------------------------------------------------------
;This code executes when a low priority interrupt occurs.

		ORG	0x0018

LowInt:	
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register
		movff	FSR0H,FSR0H_TEMP	;save FSR0H register
		movff	FSR0L,FSR0L_TEMP	;save FSR0L register
		movff	TBLPTRH,TBLPTRH_TEMP
		movff	TBLPTRL,TBLPTRL_TEMP
		movff	TABLAT, TABLAT_TEMP
		;test other interrupt flags here

		btfss	PIR1,TMR1IF	;test for Timer1 interrupt flag
		bra		LowInt1	
		btfsc	PIE1,TMR1IE	;else test if Timer3 interrupt enabled
		bra		Timer1ISR		;if so, go get received data
LowInt1:
		btfss	PIR1,RCIF	;test for RCIF receive interrupt flag
		bra		LowInt2	;if RCIF is not set, done with test
		btfsc	PIE1,RCIE	;else test if Rx interrupt enabled
		bra		GetData		;if so, go get received data
LowInt2:
		btfss	PIR1,TXIF	;test for TXIF transmit interrupt flag
		bra		LowInt3		;if TXIF is not set, done with test
		btfsc	PIE1,TXIE	;else test if Tx interrupt is enabled
		bra		PutData		;if so, go transmit data
LowInt3:
		reset			;error if no valid interrupt so reset

;		ORG     0x0300		;place code after midi code

		
;------------------------------------
;End of low priority interrupt routine restores context.

EndLowInt:	
		movff	TBLPTRH_TEMP,TBLPTRH
		movff	TBLPTRL_TEMP,TBLPTRL
		movff	TABLAT_TEMP,TABLAT
		movff	FSR0L_TEMP,FSR0L	;restore FSR0L register
		movff	FSR0H_TEMP,FSR0H	;restore FSR0H register
		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;----------------------------------------------------------------------------
;High priority interrupt routine.

HighIntCode:	
		btfss	PIR2,TMR3IF	;test for Timer3 receive interrupt flag
		bra		HighInt1	;if RCIF is not set, done with test
		btfsc	PIE2,TMR3IE	;else test if Timer3 interrupt enabled
		bra		Timer3ISR		;if so, go get received data
;can do special error handling here - an unexpected interrupt occurred 
HighInt1:	
		reset			;error if no valid interrupt so reset



;----------------------------------------------------------------------------
;Main routine checks for for reception of a <CR> and then calls a routine to
; move the data to transmit back.

Main:	
		setf	LATA	; Address strobes high
		clrf	LATB	; All outputs low
		setf	LATC	; All outputs low
	
		clrf 	TRISA 	; Config PORTA as all outputs
#ifdef ICD
		movlw	0xE0;
		movwf	TRISB
#else
		clrf 	TRISB 	; Config PORTB as all outputs
#endif
		movlw	0x80	;
		movwf 	TRISC 	; Config PORTC all outputs + tristate for serial rx
		movlw 	0x07	; init adc
		movwf	ADCON1	; init adc
		clrf 	ADCON0  ; poweroff adc
		clrf 	TRISA 	; Config PORTA again as all outputs
	
		setf	LATA	; Address strobes high
		clrf	LATB	; All outputs low
		setf	LATC	; All outputs low
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		clrf	LATA
		clrf	LATB
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		setf	LATA	; Address strobes high
		setf	LATC	; All outputs low	
	
		MOVLB 	high ExtLatAData1;
		
		clrf	ExtLatAData1;
		clrf	ExtLatAData2;
		clrf	ExtLatBData1;
		clrf	ExtLatBData2;
		clrf	ExtLatCData1;
		clrf	ExtLatCData2;
		clrf	ExtLatDData1;
		clrf	ExtLatDData2;
		clrf	ExtLatEData1;
		clrf	ExtLatEData2;
		clrf	ExtLatFData1;
		clrf	ExtLatFData2;
		clrf	ExtLatGData1;
		clrf	ExtLatGData2;
		clrf	ExtLatHData1;
		clrf	ExtLatHData2;
		clrf	ExtLatIData1;
		clrf	ExtLatIData2;
	
		setf	ExtLatAStrobeA1
		setf	ExtLatAStrobeC1
		setf	ExtLatAStrobeA2
		setf	ExtLatAStrobeC2
		
		setf	ExtLatBStrobeA1
		setf	ExtLatBStrobeC1
		setf	ExtLatBStrobeA2
		setf	ExtLatBStrobeC2

		setf	ExtLatCStrobeA1
		setf	ExtLatCStrobeC1
		setf	ExtLatCStrobeA2
		setf	ExtLatCStrobeC2

		setf	ExtLatDStrobeA1
		setf	ExtLatDStrobeC1
		setf	ExtLatDStrobeA2
		setf	ExtLatDStrobeC2

		setf	ExtLatEStrobeA1
		setf	ExtLatEStrobeC1
		setf	ExtLatEStrobeA2
		setf	ExtLatEStrobeC2

		setf	ExtLatFStrobeA1
		setf	ExtLatFStrobeC1
		setf	ExtLatFStrobeA2
		setf	ExtLatFStrobeC2

		setf	ExtLatGStrobeA1
		setf	ExtLatGStrobeC1
		setf	ExtLatGStrobeA2
		setf	ExtLatGStrobeC2

		setf	ExtLatHStrobeA1
		setf	ExtLatHStrobeC1
		setf	ExtLatHStrobeA2
		setf	ExtLatHStrobeC2

		setf	ExtLatIStrobeA1
		setf	ExtLatIStrobeC1
		setf	ExtLatIStrobeA2
		setf	ExtLatIStrobeC2

		BCF		ExtLatAStrobeC1,1
		BCF		ExtLatAStrobeC2,1

		BCF		ExtLatBStrobeC1,2
		BCF		ExtLatBStrobeC2,2

		BCF		ExtLatCStrobeC1,3
		BCF		ExtLatCStrobeC2,3

		BCF		ExtLatDStrobeA1,5
		BCF		ExtLatDStrobeA2,5

		BCF		ExtLatEStrobeA1,4
		BCF		ExtLatEStrobeA2,4

		BCF		ExtLatFStrobeA1,3
		BCF		ExtLatFStrobeA2,3

		BCF		ExtLatGStrobeA1,2
		BCF		ExtLatGStrobeA2,2
		
		BCF		ExtLatHStrobeA1,1
		BCF		ExtLatHStrobeA2,1
		
		BCF		ExtLatIStrobeA1,0
		BCF		ExtLatIStrobeA2,0


		MOVLB 	0
		CLRF	NoteCounter	
		LFSR	2,ExtLatAData1;
		
		call 	UserInit;

		lfsr  	FSR0 ,0000h
		lfsr  	FSR1 ,0000h
		rcall 	SetupMidi	;set up serial port and buffers
		rcall 	SetupTimer3
		rcall 	SetupTimer1

; TEST
;		movlw	0x20
;		movwf	MidiByte1
;		movlw	0x20
;		movwf	MidiByte2
;		call		MidiInNoteOn
;		nop
;		call		MidiInNoteOff
;		nop
; /TEST

MainLoop:	
		call	PollMidiIn;
	bra	MainLoop	;go wait for more data


;----------------------------------------------------------------------------
; Timer3 setup

SetupTimer3:
		movlw 	0x01		
		movwf 	T3CON		; timer3 on, other bits cleared
		bsf		IPR2,1		; timer3 = high priority interrupt
		bsf		PIE2,1		; timer3 interrupt enable
	return		


Timer3ISR:
#ifdef heartbeat
		bsf LATC,0;		; LATC:5 = TIMER ISR HEARTBEAT
#endif
;		movlw	0x02
;		movwf	FSR2H

;		movlw	0xFC
;		andwf	FSR2L
;		nop
		MOVFF	POSTINC2, LATB ; data on bus
			BCF	PIR2,TMR3IF	; clear interrupt flag
			MOVLW	0E0h	; 32 cycles ..
			MOVWF	TMR3L	; preset timer
			SETF	TMR3H	; preset timer
		MOVFF	POSTINC2, LATA ; set strobe
		MOVFF	POSTINC2, LATC ; set strobe
		MOVF	POSTINC2,W ;
			MOVLW	low Wrap1
			CPFSLT	FSR2L
			CLRF	FSR2L
		SETF	LATA	; clear strobe
		SETF	LATC	; clear strobe
#ifdef heartbeat
		bcf LATC,1;		; heatbeat
#endif
#ifdef simulate
	RETURN			; OUTSIDE ISR *WARNING* FOR SIMULATION ONLY
#endif
	retfie	FAST			;return and restore context
	
	ORG	0x500	

MidiInNoteOn: 
		movf	MidiByte2,0;
		bz		MidiInNoteOff;	; velocity=0 -> NOTE OFF
		movlw	high NoteNumbers;
		movwf	TBLPTRH;  page = note number table
		MOVFF 	MidiByte1,TBLPTRL
		TBLRD* 	; read into TABLAT 
		movf	TABLAT,W
		bz 		MidiInNoteOn2Proxy;
		DECF	WREG
		movwf	TBLPTRL
				; ExtLatXData1 = LatchMask OR ExtLatXData1
		MOVLW	high ExtLatAData1
		MOVWF	FSR0H
		MOVLW	high IndexToLatchPtr1
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVFF	TABLAT,FSR0L ; FSR0 points at ExtLatXData1
		MOVLW	high IndexToLatchMask
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVF	TABLAT,W
		IORWF	INDF0	
				; ExtLatXData2 = LatchMask OR ExtLatXData2
		MOVLW	high ExtLatAData1
		MOVWF	FSR0H
		MOVLW	high IndexToLatchPtr2
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVFF	TABLAT,FSR0L ; FSR0 points at ExtLatXData1
		MOVLW	high IndexToLatchMask
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVF	TABLAT,W
		IORWF	INDF0
				; initialize counter
		MOVLW	high NoteTimer
		MOVWF	FSR0H
		MOVFF	TBLPTRL,FSR0L
		MOVLW	0x05
		MOVWF	INDF0
				; done!
	BRA 	MidiInNoteOn2;		
	RETURN

MidiInNoteOn2Proxy:
		BRA MidiInNoteOn2

MidiInNoteOff:
		movlw	high NoteNumbers;
		movwf	TBLPTRH;  page = note number table
		MOVFF 	MidiByte1,TBLPTRL
		TBLRD* 	; read into TABLAT 
		movf	TABLAT,W
		bz 		MidiInNoteOff2;
		DECF	WREG
		movwf	TBLPTRL
				; ExtLatXData1 = !LatchMask AND ExtLatXData1
		MOVLW	high ExtLatAData1
		MOVWF	FSR0H
		MOVLW	high IndexToLatchPtr1
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVFF	TABLAT,FSR0L ; FSR0 points at ExtLatXData1
		MOVLW	high IndexToLatchMask
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVF	TABLAT,W
		XORLW	0xFF
		ANDWF	INDF0
				; ExtLatXData2 = !LatchMask AND ExtLatXData2
		MOVLW	high ExtLatAData1
		MOVWF	FSR0H
		MOVLW	high IndexToLatchPtr2
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVFF	TABLAT,FSR0L ; FSR0 points at ExtLatXData1
		MOVLW	high IndexToLatchMask
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVF	TABLAT,W
		XORLW	0xFF
		ANDWF	INDF0
				; initialize counter
;		MOVLW	high NoteTimer
;		MOVWF	FSR0H
;		MOVFF	TBLPTRL,FSR0L
;		SETF	INDF0
				; done!
	BRA 	MidiInNoteOff2;		
	RETURN


SetupTimer1:
		MOVLW	0x31
		MOVWF	T1CON;
		BCF		PIR1,TMR1IF
		BCF		IPR1,TMR1IP	; Timer 1 is low priority
		BSF		PIE1,TMR1IE	; Timer 1 interupt enable
		BSF		INTCON,PEIE ;
		BSF		RCON, IPEN ; interrupt priorities enabled
		BSF		PIR1,TMR1IF
	RETURN


Timer1ISR:	
Update1Timer:
		INCF	NoteCounter
		MOVF	NoteCounter,W
		SUBLW	0x49
		BNZ		NoteCounterNoWrap
		; wrap
		MOVLW	0x18
		MOVWF	NoteCounter
NoteCounterNoWrap:
		; load FSR0
		MOVLW	high NoteTimer
		MOVWF	FSR0H
		MOVFF	NoteCounter,FSR0L
		; increment and wrap NoteCounter
		movlw	0x03F	; protect
		andwf	INDF0	; protect
		DCFSNZ	INDF0
		BRA		NoteCounterExpired		
;	RETURN
		BCF		PIR1,TMR1IF
		SETF	TMR1H
		MOVLW	0xB0
		MOVWF	TMR1L
		BRA		EndLowInt

NoteCounterExpired:
		movlw	high NoteNumbers;
		movwf	TBLPTRH;  page = note number table
		MOVFF	NoteCounter,TBLPTRL
				; ExtLatXData1 = !LatchMask AND ExtLatXData1
		MOVLW	high ExtLatAData1
		MOVWF	FSR0H
		MOVLW	high IndexToLatchPtr2
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVFF	TABLAT,FSR0L ; FSR0 points at ExtLatXData1
		MOVLW	high IndexToLatchMask
		MOVWF	TBLPTRH
		TBLRD* 	
		MOVF	TABLAT,W
		XORLW	0xFF
		ANDWF	INDF0
;	RETURN
		BCF		PIR1,TMR1IF
		SETF	TMR1H
		MOVLW	0xB0
		MOVWF	TMR1L
		BRA		EndLowInt



	
MidiInCCAllNotesOff:
		MOVLB	high ExtLatAData1
		clrf	ExtLatAData1;
		clrf	ExtLatAData2;
		clrf	ExtLatBData1;
		clrf	ExtLatBData2;
		clrf	ExtLatCData1;
		clrf	ExtLatCData2;
		clrf	ExtLatDData1;
		clrf	ExtLatDData2;
		clrf	ExtLatEData1;
		clrf	ExtLatEData2;
		clrf	ExtLatFData1;
		clrf	ExtLatFData2;
		clrf	ExtLatGData1;
		clrf	ExtLatGData2;
		clrf	ExtLatHData1;
		clrf	ExtLatHData2;
		clrf	ExtLatIData1;
		clrf	ExtLatIData2;
		MOVLB  	0
	RETURN


