#define MidiChannel 9
;#define ICD
;#define MIDITHROUGHTEST


; harmal.asm
; Dec 2005
; by Johannes Taelman

		list p=18f252		;list directive to define processor
		#include <p18f252.inc>	;processor specific definitions

; #define simulate 1 ; enable simulation
; #define heartbeat 1 ; enable heatbeats
;******************************************************************************

#define	ErrorLed	LATC,	0;
#define	ClearErrorLed	BCF	ErrorLed
#define	SetErrorLed		BSF	ErrorLed

		#include "..\midi.asm"

;Configuration bits

	CONFIG	OSC = HSPLL
	CONFIG	OSCS = OFF
	CONFIG	PWRT = ON
	CONFIG	BOR = ON	
	CONFIG	BORV = 27	
	CONFIG	WDT = OFF
	CONFIG	WDTPS = 1
	CONFIG	CCP2MUX = OFF
	CONFIG	STVR = ON
	CONFIG	LVP = OFF

;----------------------------------------------------------------------------
; Variables
; memory map 
;----------------------------------------------------------------------------

		CBLOCK	0x000
		WREG_TEMP		;to save WREG during interrupt
		STATUS_TEMP		;to save STATUS during interrupt
		BSR_TEMP		;to save BSR during interrupt
		FSR0H_TEMP		;to save FSR0H during interrupt
		FSR0L_TEMP		;to save FSR0L during interrupt
		TBLPTRH_TEMP
		TBLPTRL_TEMP
		TABLAT_TEMP
		FSR0H_SHADOW		;to save FSR0H during high interrupt
		FSR0L_SHADOW		;to save FSR0L during high interrupt
		TmpM			; non-ISR scratch
		TmpT			; non-ISR scratch		
		ENDC
;----------------------------------------------------------------------------
; end of memory map
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;This code executes when a reset occurs.

		ORG     0x0000		;place code at reset vector

ResetVector:			bra	Main		;go to beginning of program

;----------------------------------------------------------------------------
;This code executes when a high priority interrupt occurs.

		ORG	0x0008

HighInt:
		reset			;error if no valid interrupt so reset

;----------------------------------------------------------------------------
;This code executes when a low priority interrupt occurs.

		ORG	0x0018

LowInt:	
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register
		movff	FSR0H,FSR0H_TEMP	;save FSR0H register
		movff	FSR0L,FSR0L_TEMP	;save FSR0L register
		movff	TBLPTRH,TBLPTRH_TEMP
		movff	TBLPTRL,TBLPTRL_TEMP
		movff	TABLAT, TABLAT_TEMP
		;test other interrupt flags here

;		btfss	PIR1,TMR1IF	;test for Timer1 interrupt flag
;		bra		LowInt1	
;		btfsc	PIE1,TMR1IE	;else test if Timer3 interrupt enabled
;		bra		Timer1ISR		;if so, go get received data
LowInt1:
		btfss	PIR1,RCIF	;test for RCIF receive interrupt flag
		bra		LowInt2	;if RCIF is not set, done with test
		btfsc	PIE1,RCIE	;else test if Rx interrupt enabled
		bra		GetData		;if so, go get received data
LowInt2:
		btfss	PIR1,TXIF	;test for TXIF transmit interrupt flag
		bra		LowInt3		;if TXIF is not set, done with test
		btfsc	PIE1,TXIE	;else test if Tx interrupt is enabled
		bra		PutData		;if so, go transmit data
LowInt3:
		reset			;error if no valid interrupt so reset

;		ORG     0x0300		;place code after midi code

		
;------------------------------------
;End of low priority interrupt routine restores context.

EndLowInt:	
		movff	TBLPTRH_TEMP,TBLPTRH
		movff	TBLPTRL_TEMP,TBLPTRL
		movff	TABLAT_TEMP,TABLAT
		movff	FSR0L_TEMP,FSR0L	;restore FSR0L register
		movff	FSR0H_TEMP,FSR0H	;restore FSR0H register
		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie



;----------------------------------------------------------------------------
;Main routine checks for for reception of a <CR> and then calls a routine to
; move the data to transmit back.

Main:	
		setf	LATA	; Address strobes high
		clrf	LATB	; All outputs low
		setf	LATC	; All outputs low
	
		clrf 	TRISA 	; Config PORTA as all outputs
#ifdef ICD
		movlw	0xE0;
		movwf	TRISB
#else
		clrf 	TRISB 	; Config PORTB as all outputs
#endif
		movlw	0x80	;
		movwf 	TRISC 	; Config PORTC all outputs + tristate for serial rx
		movlw 	0x07	; init adc
		movwf	ADCON1	; init adc
		clrf 	ADCON0  ; poweroff adc
		clrf 	TRISA 	; Config PORTA again as all outputs

		call	LedGlowIn
;		call	LedGlowOut defect

		call 	MidiInCCMotorOff		
	
; init PWM
		movlw	0x7E;
		movwf	PR2
		clrf	CCPR1L
		movlw	0x0F;
		movwf	CCP1CON
		movlw 	0x05
		movwf 	T2CON
		movlw	0x7F;
		movwf	CCPR1L

		lfsr  	FSR0 ,0000h
		lfsr  	FSR1 ,0000h
		rcall 	SetupMidi	;set up serial port and buffers

MainLoop:	
		call	PollMidiIn;
	bra	MainLoop	;go wait for more data


;----------------------------------------------------------------------------
; Timer3 setup

;SetupTimer3:
;		movlw 	0x01		
;		movwf 	T3CON		; timer3 on, other bits cleared
;		bsf		IPR2,1		; timer3 = high priority interrupt
;		bsf		PIE2,1		; timer3 interrupt enable
;	return		

;Timer3ISR:
;	retfie	FAST			;return and restore context
		
	ORG 0x500

MidiInCCAllNotesOff:
	RETURN

MidiInCtrlChange:
		movlw	0x7B; 			CC NotesOff
		subwf	MidiByte1,0;
		bz		MidiInCCAllNotesOff;

		movlw	0x07; 			CC Volume
		subwf	MidiByte1,0;
		bz		MidiInCCVolume;

		movlw	0x42; 			CC motor control aan/uit
		subwf	MidiByte1,0;
		bz		MidiInCCMotorOnOff;

	return
	
MidiInCCVolume:
		movf	MidiByte2,W
		xorlw	0x7F;
		movwf	CCPR1L
	return

MidiInCCMotorOnOff:
		movf	MidiByte2,W
		andlw	0x7F;
		bz		MidiInCCMotorOff;
		bcf		LATC,3;
	return

MidiInCCMotorOff:
		bsf		LATC,3;
	return	

MidiInProgramChange:
		movf	MidiByte1,W
		andlw	0xF0;
		bnz		irrelevantPGC;
		movf	MidiByte1,W
		movwf	LATA;
	return

		
irrelevantPGC:
	return

MidiInNoteOn
MidiInNoteOff
MidiInChannelPressure:



MidiInRTReset:
MidiInRTActiveSense:
MidiInRTStop:
MidiInRTContinue:
MidiInRTStart:
MidiInRTTick:
MidiInRTClock:
MidiInTuneRequest:
MidiInAfterTouch:
MidiInPitchWheel:
MidiInSysExStart:
MidiInSysExByte:
MidiInSysExEnd:
	RETURN


	END
